#!/bin/sh

# Partition a fresh hard disk with optimal alignment, including
# a LUKS volume and filesystem inside.

set -e

disk=$1; shift
name=$1; shift
cipher=$1; shift
keysize=$1; shift
filesystem=$1; shift

# I know this would be shorter with Bash syntax. This here
# is more obvious and I like basic scripts to use the basic
# shell syntax.
if test -z "$name"; then
  name=backup
fi
if test -z "$cipher"; then
  cipher=twofish-xts-plain
fi
if test -z "$keysize"; then
  keysize=512
fi
if test -z "$filesystem"; then
  filesystem=ext4
fi

parted -s "$disk" mktable gpt
parted -s -a optimal "$disk" mkpart allofit 0% 100%
cryptsetup luksFormat -c "$cipher" -s "$keysize" "${disk}1"
cryptsetup luksOpen "${disk}1" "$name"
# Optionally provide commands to create filesystem as script parameters
# after giving "custom" as filesystem. Otherwise, it must be something
# supported by mke2fs.
if test "$filesystem" = custom; then
  "$@" "/dev/mapper/$name"
else
  mke2fs -t "$filesystem" -L "$name$(date +%Y%m%d)" -m 0 "/dev/mapper/$name"
fi
cryptsetup luksClose "$name"
