# I don't have a good name yet. It's a module that contains the
# routines that find files on my mirror.
package MirrorFind;

# I even don't do proper file path handling. This script runs on unix. A
# slash means what it is supposed to mean. Actually, that's the proper
# way;-)

# In future, one might employ an index. Let's see if performance indeed
# matters, first. And also don't prematurely introduce something that
# might even be slower than the filesystem action.

use strict;
use File::Basename qw(dirname basename);

# Fixing the location of the whole mirror stuff (files, urls, scripts) for now.
# Less hassle.
# In case anyone else wants to host such  a mirror, too, one can change that.
our $root = '/home/mirror.sobukus.de';
our $debug = 0;

# Code that modifes path if it does not exist.
# You could append subroutines here ...
our @modpath = (sub { $_[0] =~ s,^files/sourcemage/(grimoire/|)fallback/,, });

our @chapters = qw(src cpan gnu gnome grimoire linux_x86-64 sourcemage);
my $chapterx_txt = '^('.join('|',@chapters).')/(.*)$';
my $chapterx = qr/$chapterx_txt/;

sub candidate_list
{
	my $path = shift;
	my @cand = ("files/$path");
	for(@chapters)
	{
		push(@cand, "files/$_/$path", "files/$_/_unsorted/$path");
	}

	return @cand;
}

sub find_full_path
{
	my $path = shift;
	return undef
		unless $path;

	# 0. Realise early that the path exists as-is.
	my $fullpath = "$root/$path";
	return $fullpath
		if -e $fullpath;

	# Before guessing further: Mangle the path with provided code.
	for(@modpath){ $_->($path) }

	# 1. Simple guesses
	print STDERR "Step 1: candidates\n"
		if $debug;
	for my $candidate
	(	# "$path" no ... this is only good for circular redirects
		candidate_list($path)
	)
	{
		my $fullpath = "$root/$candidate";
		print STDERR "candidate: $fullpath\n"
			if $debug;
		return $fullpath
			if -e $fullpath;
	}
	# Now look closer at the file name itself.
	my @prefix = @chapters;
	if($path =~ $chapterx)
	{
		@prefix = ($1);
		$path = $2;
	}
	# The $path should be a plain filename now (would be found already otherwise).
	print STDERR "Step 2: path-less filenames\n"
		if $debug;
	return undef if $path =~ m:/:;
	for my $prefix (@prefix)
	{
		if(-e "$root/files/$prefix/.use_index" && open(my $idx, '<', "$root/files/$prefix/.index"))
		{
			#print STDERR "using .index\n";
			# Try to be quicker by looking at an index.
			# This is for file systems that are really slow
			# on metadata operations.
			# Calling basename() here slows things down _a_ _lot_!
			# Should not hurt to omit it.
			while(<$idx>)
			{
				if(index($_, $path) >= 0 and chomp() and basename($_) eq $path)
				{
					return "$root/files/$prefix/$_";
				}
			}
		}
		else
		{
			#print STDERR "looking for $root/files/$prefix/*/$path\n";
			require File::Glob;
			# GLOB_NOSORT is an option to make it faster ... do we want
			# the determinism of sorting?
			if((my $path = File::Glob::bsd_glob("$root/files/$prefix/*/$path")))
			{
				return $path;
			}
		}
	}
	# Might be part of a directory name.
	for my $prefix (@prefix)
	{
		#print STDERR "looking for $root/files/$prefix/*$path*\n";
		require File::Glob;
		# GLOB_NOSORT is an option to make it faster ... do we want
		# the determinism of sorting?
		if((my $path = File::Glob::bsd_glob("$root/files/$prefix/*$path*")))
		{
			return $path;
		}
	}
	# Look deeper: A directory containing the wanted file.
	for my $prefix (@prefix)
	{
		my @path;
		#print STDERR "looking for $root/files/$prefix/*/*$path*\n";
		if(-e "$root/files/$prefix/.use_index" && open(my $idx, '<', "$root/files/$prefix/.index"))
		{
			#print STDERR "using .index\n";
			# Try to be quicker by looking at an index.
			# This is for file systems that are really slow
			# on metadata operations.
			# Calling basename() here slows things down _a_ _lot_!
			# Should not hurt to omit it.
			while(<$idx>)
			{
				(chomp() and push(@path, "$root/files/$prefix/$_"))
					if(index($_, $path) >= 0);
				last
					if @path == 2; # 2 are enough
			}
		}
		else
		{
			require File::Glob;
			# GLOB_NOSORT is an option to make it faster ... do we want
			# the determinism of sorting?
			@path = File::Glob::bsd_glob("$root/files/$prefix/*/*$path*");
		}
		# We just assume multiple occurences in the same directory. The main
		# point is not giving a direct download link if it is not unique.
		if(@path)
		{
			return @path == 1 ? $path[0] : dirname($path[0]);
		}
	}
	return undef;
}

1;

